{%- if ipv6_override is not defined %}
{%- set ipv6_override = false %}
{%- endif %}
heat_template_version: wallaby

description: >
  Creates a port on the {{network.name}} network, using a map of IPs per role.
  Each role has a map of IPs in <Role>IPs parameters, with a list of IPs by
  network (lower_name or lower case). For example:
  ControllerIPs:
    external:
    - 1.2.3.4 # First controller
    - 1.2.3.5 # Second controller

parameters:
  {{network.name}}NetName:
    description: The name of the {{network.name_lower}} network.
    default: {{network.name_lower}}
    type: string
  PortName:
    description: Name of the port
    default: ''
    type: string
  DnsName:
    description: DNS name of the port
    default: ''
    type: string
  FixedIPs:
    description: >
        Control the IP allocation for the VIP port. E.g.
        [{'ip_address':'1.2.3.4'}]
    default: []
    type: json
  ControlPlaneIP: # Here for compatibility with noop.yaml
    description: IP address on the control plane
    default: ''
    type: string
  ControlPlaneSubnetCidr: # Here for compatibility with noop.yaml
    description: >
      The subnet CIDR of the control plane network. (The parameter is
      automatically resolved from the ctlplane subnet's cidr attribute.)
    type: string
    default: ''
  ControlPlaneNetwork: # Here for compatibility with ctlplane_vip.yaml
    description: The name of the undercloud Neutron control plane
    default: ctlplane
    type: string
  ControlPlaneSubnet: # Here for compatibility with ctlplane_vip.yaml
    description: The name of the undercloud Neutron control plane subnet
    default: ctlplane-subnet
    type: string
  IPPool: # Set in <Role>IPs map, see environments/ips-from-pool-all.yaml
    default: {}
    type: json
  NodeIndex: # First node in the role will get first IP, and so on...
    default: 0
    type: number
  {{network.name}}NetCidr:
{%- if network.ipv6|default(false)  or ipv6_override %}
    default: "{{network.ipv6_subnet|default(network.ip_subnet|default(""))}}"
{%- else %}
    default: "{{network.ip_subnet|default("")}}"
{%- endif %}
    description: Cidr for the {{network.name_lower}} network.
    type: string
  DefaultRoute: # Here for compatibility
    description: >
      Whether this interface is used for the default route
    type: boolean
    default: false
  IsVirtualIP:  # Here for compatibility
    default: false
    type: boolean
  Role: # Here for compatibility
    description: >
      The TripleO Role Name
    type: string
    default: ''

outputs:
  ip_address:
    description: {{network.name}} network IP
    value: {get_param: [IPPool, {get_param: {{network.name}}NetName}, {get_param: NodeIndex}]}
  ip_address_uri:
    description: {{network.name}} network IP (with brackets for IPv6 URLs)
    value:
      str_replace:
        template:
          make_url:
            host: {get_param: [IPPool, {get_param: {{network.name}}NetName}, {get_param: NodeIndex}]}
        params:
          //: ''
  ip_subnet:
    description: IP/Subnet CIDR for the {{network.name}} network IP
    value:
      list_join:
      - ''
      - - {get_param: [IPPool, {get_param: {{network.name}}NetName}, {get_param: NodeIndex}]}
        - '/'
        - {str_split: ['/', {get_param: {{network.name}}NetCidr}, 1]}
