#jinja2: trim_blocks:False
network_config:
- type: ovs_bridge
  name: br-ctlplane
  mtu: 1450
  use_dhcp: false
  dns_servers:
  
  - 192.168.122.10
  
  - 192.168.122.10
  domain: []
  addresses:
  - ip_netmask: 192.168.122.100/24
  - ip_netmask: 192.168.122.100/32
  - ip_netmask: 192.168.122.99/32
  
  members:
  - type: interface
    name: nic2
    mtu: 1450
    # force the MAC address of the bridge to this interface
    primary: true
  # external
  - type: vlan
    mtu: 1450
    vlan_id: 44
    addresses:
    - ip_netmask: 172.21.0.100/24
    - ip_netmask: 172.21.0.2/32
    routes: []
  # internal
  - type: vlan
    mtu: 1450
    vlan_id: 20
    addresses:
    - ip_netmask: 172.17.0.100/24
    - ip_netmask: 172.17.0.2/32
  
    routes:
      - ip_netmask: 172.17.1.0/24
        next_hop: 172.17.0.100
  
  # storage
  - type: vlan
    mtu: 1450
    vlan_id: 21
    addresses:
    - ip_netmask: 172.18.0.100/24
    - ip_netmask: 172.18.0.2/32
  
    routes:
      - ip_netmask: 172.18.1.0/24
        next_hop: 172.18.0.100
  
  # storage_mgmt
  - type: vlan
    mtu: 1450
    vlan_id: 23
    addresses:
    - ip_netmask: 172.20.0.100/24
    - ip_netmask: 172.20.0.2/32
  
    routes:
      - ip_netmask: 172.20.1.0/24
        next_hop: 172.20.0.100
  
  # tenant
  - type: vlan
    mtu: 1450
    vlan_id: 22
    addresses:
    - ip_netmask: 172.19.0.100/24
  
    routes:
      - ip_netmask: 172.9.1.0/24
        next_hop: 172.19.0.100
  
