---
network_config:
- type: interface
  name: nic1
  use_dhcp: false
  dns_servers: {{ ctlplane_dns_nameservers }}
  addresses:
  - ip_netmask: {{ ctlplane_ip }}/NEED MANUAL CONVERSION: {''get_param'': ''ControlPlaneEdge1SubnetCidr''}
  routes: {{ [{'default': True, 'next_hop': "NEED MANUAL CONVERSION: {''get_param'': ''ControlPlaneEdge1DefaultRoute''}"}, [{'ip_netmask':
    '192.168.254.0/24', 'next_hop': '192.168.24.1'}], [{'ip_netmask': '169.254.169.254/32', 'next_hop':
    "NEED MANUAL CONVERSION: {''get_param'': ''Edge1EC2MetadataIp''}"}]] | flatten | unique }}
- type: ovs_bridge
  name: br-ex
  use_dhcp: false
  members:
  - type: interface
    name: nic2
    primary: true
  - type: vlan
    vlan_id: {{ storageedge1_vlan_id }}
    addresses:
    - ip_netmask: {{ storageedge1_ip }}/{{ storageedge1_cidr }}
    routes:
    - ip_netmask: NEED MANUAL CONVERSION: {''get_param'': ''StorageSupernet''}
      next_hop: {{ storageedge1_gateway_ip }}
  - type: vlan
    vlan_id: {{ internal_apiedge1_vlan_id }}
    addresses:
    - ip_netmask: {{ internal_apiedge1_ip }}/{{ internal_apiedge1_cidr }}
    routes:
    - ip_netmask: NEED MANUAL CONVERSION: {''get_param'': ''InternalApiSupernet''}
      next_hop: {{ internal_apiedge1_gateway_ip }}
  - type: vlan
    vlan_id: {{ tenantedge1_vlan_id }}
    addresses:
    - ip_netmask: {{ tenantedge1_ip }}/{{ tenantedge1_cidr }}
    routes:
    - ip_netmask: NEED MANUAL CONVERSION: {''get_param'': ''TenantSupernet''}
      next_hop: {{ tenantedge1_gateway_ip }}
